/* 
 * Copyright (c) 2004
 * Bankenes BetalingsSentral AS (BBS)
 * Organisation Number N- 975 946 231
 * Haavard Martinsens vei 54
 * N- 0045 OSLO
 * Norway
 */

#ifndef BIDCSERVER_SERVERKERNEL_H
#define BIDCSERVER_SERVERKERNEL_H

#include <stdlib.h>
#include "structures.h"


#ifdef __cplusplus
extern "C" {
#endif 

/* The BankID Server API */
/* -------------------------------------------------------------------- */

/** bid_initialize is the first method a merchant must call in order to initialize the
    BankID C Server. The method initializes internal structures.
    @return Returns zero if the initialization of BankID C Server was successfull.
            If an error occurs during initialization an error code is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_initialize();

/** bid_initialize is the first method a merchant must call in order to initialize the
    BankID C Server. The method initializes internal structures.
    @param pkcs11driver The full path to the PKCS#11 implementation to use.
    @return Returns zero if the initialization of BankID C Server was successfull.
            If an error occurs during initialization an error code is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_hsminitialize( const char* pkcs11driver);

/** bid_finalize cleans up the BankID resources. It is called after all sessions are closed.
    @return Returns zero if the uninitialization of BankID C Server was successfull.
            If an error occurs during initialization an error code is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_finalize();

/** bid_opensession must be called for every merchant that wants to create a merchantsession
    towards the library. When BID_OpenSession returns a context is created which must be used in
    all future calls for services.
    @param sessioncontext This is a reference to a variable where BankID C Server will store the context for the merchant
    @param keylocation This must be the full path to the merchants keystore file.
    @param password This is the password for the merchants keystore file.
    @param configlocation This must contain the full path to the merchants configuration file, or NULL if not supplied
    @param webaddress The web addresses and ip addresses used for MITM.
    @param tagcache The full path to where to store cached tags.
    @param truststore The full path to where the trusted ca certificates are located for ssl communication
    @param grantedpolicies List of policyOIDs which are granted to be used with BankID Server.
    @param proxyserver The ip address of an http proxy server.
    @param proxyport The port of the http proxy server.    

    @return Returns zero if BankID C Server was able to open a session for the merchant.
            If an error occurs during session creation an error code is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_opensession( int* sessioncontext,
                            const char* keylocation,
                            const char* password,
                            const char* configlocation,
                            const char* webaddress,
                            const char* tagcache,
                            const char* trustedstore,
                            const char* grantedpolicies,
                            const char* proxyserver,
                            const char* proxyport);


/** bid_opensession must be called for every merchant that wants to create a merchantsession
    towards the library. When BID_OpenSession returns a context is created which must be used in
    all future calls for services.
    @param sessioncontext This is a reference to a variable where BankID C Server will store the context for the merchant
    @param keylocation This must be the full path to the merchants keystore file.
    @param password This is the password for the merchants keystore file.
    @param configlocation This must contain the full path to the merchants configuration file, or NULL if not supplied
    @param webaddress The web addresses and ip addresses used for MITM.
    @param tagcache The full path to where to store cached tags.
    @param truststore The full path to where the trusted ca certificates are located for ssl communication
    @param grantedpolicies List of policyOIDs which are granted to be used with BankID Server.
    @param proxyserver The ip address of an http proxy server.
    @param proxyport The port of the http proxy server.    
    @param pkcs11password The password to the HSM token wheer the private keys reside
    @param slot The slotid where the private keys is stored.

    @return Returns zero if BankID C Server was able to open a session for the merchant.
            If an error occurs during session creation an error code is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_hsmopensession( int* sessioncontext,
                               const char* keylocation,
                               const char* password,
                               const char* configlocation,
                               const char* webaddress,
                               const char* tagcache,
                               const char* trustedstore,
                               const char* grantedpolicies,
                               const char* proxyserver,
                               const char* proxyport,
                               const char* pkcs11password,
                               const int slot);


/** BID_RAHSMOpenSession must be called for every bank that wants to create an RA session
    towards the library where their private keys are stored in an HSM. 
    BID_RAHSMOpenSession is called for banks who need to run BankID Server in RA mode.
    RA moe means that the server is used to sign and build a dsig xml which is further sent to the ODS system.
    When BID_HSMOpenSession returns a context is created which must be used in
    all future calls for services.
    @param sessioncontext This is a reference to a variable where BankID C Server will store the context for the merchant
    @param keylocation This must be the full path to the merchants keystore file.
    @param password This is the password for the merchants keystore file.
    @param pkcs11password The password of the HSM where the keys reside
    @param slot The slotid where the private keys is stored.

    @return Returns zero if BankID C Server was able to open a session for the bank.
            If an error occurs during session creation an error code is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_rahsmopensession( int* sessioncontext,
                                 const char* keylocation,
                                 const char* password,
                                 const char* pkcs11password,
                                 const int slot);



/** BID_CloseSession must be called for every merchant that wants to take down a merchantsession
    towards the library. When BID_CloseSession returns *no* more actions must be taken towards
    the library with the closes dession handle..
    @param sessioncontext This is the reference to the merchant's session
    @return Returns zero if BankID C Server was able to close a session for the merchant.
            If an error occurs during session creation an error code is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int  bid_closesession(const int sessioncontext);



/** bid_signdata creates a PKCS#7 signature on behalf of the merchant.
    @param sessioncontext This is the context handle for the current merchant
    @param data The data to be signed.
    @param datalen The length data to be signed.
    @param pkcs7 This is a reference to a pointer where BankID C Server will allloate memory and store the PKCS#7 structure in BASE64 format.
    @return Returns zero if BankID C Server was able to create a signature for the merchant.
            If an error occurs during the signing operation an error code is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int  bid_signdata( const int sessioncontext,
                          const unsigned char* data,
                          const size_t datalen,
                          char** pkcs7);


/** bid_signxml creates a PKCS#7 signature over an XSL stylesheet and an XML
    @param sessioncontext This is the context handle for the current merchant
    @param xml The xml that should be signed.
    @param xmllen The length of the xml that should be signed.
    @param xsl The xml that should be signed.
    @param xsllen The length of the xsl that should be signed.
    @param pkcs7 This is a reference to a pointer where BankID C Server will allloate memory and store the PKCS#7 structure in BASE64 format.
    @param data This is a reference to a pointer where BankID C Server will allloate memory and store the the data that was actually signed.
    @return Returns zero if BankID C Server was able to create a signature for the merchant.
            If an error occurs during the signing operation an error code is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_signxml( const int sessioncontext,
                        const unsigned char* xml,
                        const size_t xmllen,
                        const unsigned char* xsl,
                        const size_t xsllen,
                        char** pkcs7,
                        char** data);


/** bid_signdatap1 creates a PKCS#1 signature on behalf of the merchant.
    @param sessioncontext This is the context handle for the current merchant
    @param data The data to be signed. This can be a usual text or just raw bytes.
    @param datalen The length of the data to be signed
    @param pkcs1 This is a reference to a pointer where BankID C Server will allloate memory and store the PKCS#1 structure in BASE64 format.
    @return Returns zero if BankID C Server was able to create a signature for the merchant.
            If an error occurs during the signing operation an error code is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int  bid_signdatap1(const int sessioncontext,
                           const unsigned char* data,
                           const size_t datalen,
                           char** pkcs1);


/** bid_createxmldsig creates an enveloping XMLDSIG structure.
    @param sessioncontext This is the context handle for the current merchant
    @param c14nxmlorder A c14n canonicalized ODS order wrapped in an <Object> element.
    @param xmldsig This is a reference to a pointer where BankID C Server will alloate memory and store the XMLDSIG structure.
    @param certexpiresin This is a reference to a pointer where BankID C Server will store how many days it is until the certificate expires.
    @return Returns zero if BankID C Server was able to create an XMLDSIG for the merchant
            If an error occurs during the operation an error code is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_createxmldsig(const int sessioncontext,
                             const char* c14nxmlorder,
                             char** xmldsig,
                             int* certexpiresin);


/** bid_verifydata verifies a PKCS#7 structure against incoming data.
    @param sessioncontext This is the context handle for the current merchant
    @param pkcs7 This is a BASE64 encoded PKCS#7 structure to verify.
    @param data The BASE64 encoded data that apparently is signed when the PKCS#7 signature was created
                This can be a usual string or raw bytes.
    @param datalen The length of the data that apparently is signed when the PKCS#7 signature was created
                This can be a usual string or raw bytes.

    @return Returns zero if BankID C Server was able to verify the PKCS#7 structure successfully.
            If an error occurs during the verification process an error code is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_verifydata( const int sessioncontext,
                           const char* pkcs7,
                           const unsigned char* data,
                           const size_t datalen);


/** bid_hashverifydata verifies a PKCS#7 structure against incoming hashed data.
    @param sessioncontext This is the context handle for the current merchant
    @param pkcs7 This is a BASE64 encoded PKCS#7 structure to verify.
    @param hash The BASE64 encoded hash of the data that apparently is signed when the PKCS#7 signature was created
    @param auth If false the require sign key usages, else auth keyusages.
    
    @return Returns zero if BankID C Server was able to verify the PKCS#7 structure successfully.
            If an error occurs during the verification process an error code is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_hashverifydata(const int sessioncontext,
                              const char* pkcs7,
                              const char* hash,
                              const int auth);


/** bid_authenticateclient verifies a PKCS#7 retrieved from the client during the authentication process.
    @param sessioncontext This is the context handle for the current merchant
    @param pkcs7 This is a BASE64 encoded PKCS#7 structure to verify.
    @param serverchallenge The BASE64 encoded data that apparently is signed when the PKCS#7 signature was created.
                           During an authentication procedure the signed data is the challenge created earlier by the BankID C Server
                           using BID_AuthenticateServer.
    
    @return Returns zero if BankID C Server was able to authenticate the clients signature against the serverchallenge.
            If an error occurs during the client authentication process an error code is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_authenticateclient(const int sessioncontext,
                                  const char* pkcs7,
                                  const char* serverchallenge);

/** bid_authenticateserver creates a signature on behalf of the server that is proprietary to the authentication mechanism.
    @param sessioncontext This is the context handle for the current merchant
    @param pkcs7 This is a reference to a pointer where BankID C Server will allocate memory and fill in a BASE64 encoded PKCS#7
    @param serverchallenge This is a reference to a pointer where BankID C Server will allocate memory and fill in a BASE64 encoded generated serverchallenge
    @param clientchallenge This is an incoming BASE64 encoded clientchallenge generated by the client BankID Applet. The signature is created based on 
                           these data.
    @param authenticationheader The authentication header received from the COI if the BankID operation is triggered by a mobile phone
    @param prepareText The prepareText value received from COI if the BankID operation is triggered by a mobile phone.

    @return Returns zero if BankID C Server was create a signature over the clientchallenge and generate a serverchallenge.
            If an error occurs during the server authentication process an error code is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_authenticateserver(const int sessioncontext, char** pkcs7, char** serverchallenge, const char* clientchallenge, const char* authenticationheader, const char* prepareText);


/** bid_hashdata creates a hash of the incoming data. The hashalgorithm to use is specified by the hashalgorithm parameter.
    @param sessioncontext This is the context handle for the current merchant
    @param hash This is a reference to a pointer where BankID C Server will allocate memory and fill in the digest of the incoming data.
    @param hashalgorithm This is either SHA1 or MD5, which are defined in this headerfile.
    @param data This is the BASE64 encoded data that BankID C Server must create a digest of.

    @return Returns zero if BankID C Server was able to create a digest of the incoming data.
            If an error occurs during the digest operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_hashdata( const int sessioncontext,
                         char** hash,
                         const int hashalgorithm,
                         const char* data);


/** bid_getrandom creates the specified length of random bytes.
    @param sessioncontext This is the context handle for the current merchant
    @param rand This is a reference to a pointer where BankID C Server will allocate memory and fill in the BASE64 encoded random data.
    @param randlen This parameter specifies how many random bytes that must be generated.

    @return Returns zero if BankID C Server was able to create the requested number of random bytes.
            If an error occurs during the random operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_getrandom( const int sessioncontext,
                          char** rand,
                          const size_t randlen );


/** bid_base64encode basically BASE64 encodes the incoming data.
    @param sessioncontext This is the context handle for the current merchant
    @param in This is the incoming data to encode
    @param inlen This is the length of the incoming data
    @param out This is a reference to a pointer where BankID C Server will allocate memory and fill in the BASE64 encoded data.

    @return Returns zero if BankID C Server was able to BASE64 encode the data
            If an error occurs during the encoding operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_base64encode( const int sessioncontext,
                             const unsigned char* in,
                             const size_t inlen,
                             char** out);


/** bid_base64decode basically BASE64 decodes the incoming data.
    @param sessioncontext This is the context handle for the current merchant
    @param in This is the incoming BASE64 encoded data to decode.
    @param out This is a reference to a pointer where BankID C Server will allocate memory and fill in the BASE64 decoded data
    @param outlen This is the length of the BASE64 decoded data.

    @return Returns zero if BankID C Server was able to BASE64 decode the data
            If an error occurs during the decode operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_base64decode( const int sessioncontext,
                             const char* in,
                             unsigned char** out,
                             size_t* outlen);


/** bid_removecrlf is basically a utility method for removing "\r" and "\n" characters in the incoming string.
    @param sessioncontext This is the context handle for the current merchant
    @param in This is the incoming string to strip for "\r" and "\n" characters.
    @param out This is a reference to a pointer where BankID C Server will allocate memory and fill the incoming string stripped for "\r" and "\n"

    @return Returns zero if BankID C Server was able to strip the incoming data for "\r" and "\n"
            If an error occurs during the stripping operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_removecrlf( const int sessioncontext,
                           const char* in,
                           char** out );


/** bid_getpkcs7info is a method for retrieving the specified information item from a PKCS#7 structure.
    @param sessioncontext This is the context handle for the current merchant
    @param pkcs7 This a BASE64 encoded PKCS#7 structure to retrieve information from.
    @param infoitem This is the requested information item. The valid items are:
                    ISSUERNAME, SUBJECTNAME, VALIDFROMDATE, VALIDTODATE, VERSIONNUMBER, SERIALNUMBER, 
                    KEYALGORITHM, KEYSIZE, UNIQUEID, ORIGINATOR, BANKNAME, DATEOFBIRTH, POLICYOID
    @param out This is a reference to a pointer where BankID C Server will allocate memory and fill the requested dataitem.

    @return Returns zero if BankID C Server was able to retrieve the information from the PKCS#7
            If an error occurs during the information retrieval operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_getpkcs7info( const int sessioncontext,
                             const char* pkcs7,
                             const int infoitem,
                             char** out);

/** bid_setinfoitem is used by the merchant to store key/value pairs in its own session. The key/value pairs will 
    reside in the merchants session until the merchant itself calls BID_RemoveInfoItems().
    @param sessioncontext This is the context handle for the current merchant
    @param key This is the key to use for the value parameter.
    @param value This is the value to store under the corresponding key.

    @return Returns zero if BankID C Server was able to store the key/value pair in the merchant session.
            If an error occurs during the insertion operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_setinfoitem( const int sessioncontext,
                            const char* key,
                            const char* value );

/** bid_getinfoitem is used by the merchant to retrieve a value stored in the merchants session under the corresponding key.
    @param sessioncontext This is the context handle for the current merchant
    @param key This is the key for where to find the value.
    @param value This is a reference to a pointer where BankID C Server will allocate memory and fill in the value for the requested key.

    @return Returns zero if BankID C Server was able to retrieve the value parameter in the merchant session.
            If an error occurs during the get operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_getinfoitem( const int sessioncontext,
                            const char* key,
                            char** value );

/** bid_removeinfoitems will clear all stored key/value parameters for the merchant session.
    @param sessioncontext This is the context handle for the current merchant

    @return Returns zero if BankID C Server was able to clear the key/value storage for the merchant session
            If an error occurs during the clear operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_removeinfoitems( const int sessioncontext );


/** bid_getcertstatus accepts a PKCS#7 structure, retrieves the signing certificate and requests a certificate status 
    towards the Validation Authority.
    @param sessioncontext This is the context handle for the current merchant
    @param pkcs7 This is the BASE64 encoded signature in which the certificate in request resides.
    @param ocspresponse This is a reference to a pointer where BankID C Server will allocate storage and fill in the OCSP response from the VA.
                        If the parameter is NULL, the OCSP response will not be supplied.

    @return Returns zero if BankID C Server successfully called getcertstatus.
            If an error occurs during the getcertstatus operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_getcertstatus(  const int sessioncontext, 
                               const char* pkcs7,
                               char** ocspresponse);


/** bid_getowncertstatus will do a Validation Authority lookup on the merchants own signing certificate.
    @param sessioncontext This is the context handle for the current merchant
    @param ocspresponse This is a reference to a pointer where BankID C Server will allocate storage and fill in the OCSP response from the VA.
                        If the parameter is NULL, the OCSP response will not be supplied.

    @return Returns zero if BankID C Server successfully called getcertstatus.
            If an error occurs during the getcertstatus operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_getowncertstatus( const int sessioncontext,
                                    char** ocspresponse);


/** bid_createsdo creates a Signed Data Object that is a mutual contract between a merchant and an end user.
    The data itself will not be embedded in the SDO. The merchant must call BID_SDOAddData to embed the data in the SDO.
    @param sessioncontext This is the context handle for the current merchant
    @param sdo This a reference to a pointer to a BID_SDO structure where BankID C Server will allocate memory and fill in the SDO.
    @param data This is the BASE64 encoded data (the contract) itself.
    @param datatype The datatype of the signed data This must be either SDODATA_TEXT or SDODATA_PDF.
    @param datadescription The datadescription of the signed data.
    @param clientpkcs7 This is a BASE64 encoded client PKCS#7 apparently containing a signature over the incoming data.
    @param serverpkcs7 This is a BASE64 encoded server PKCS#7 apparently containing a signature over the incoming data.
    @param clientocspresponse This is a BASE64 encoded OCSP response from an earlier BID_GetCertStatus request for the clients certificate.
                              Set the parameter to NULL if no OCSP response is supplied
                              If supplied and all constraints are satisfied no VA lookup on client certificate will be done.
    @param serverocspresponse This is a BASE64 encoded OCSP response from an earlier BID_GetCertStatus request for the server certificate.
                              Set the parameter to NULL if no OCSP response is supplied
                              If supplied and all constraints are satisfied no VA lookup on server certificate will be done.

    @return Returns zero if BankID C Server was able to create the Signed Data Object.
            If an error occurs during the contract building operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_createsdo( const int sessioncontext,
                          BID_SEIDSDO** sdo,
                          const char* data,
                          const char* datatype,
                          const char* datadescription,
                          const char* clientpkcs7,
                          const char* serverpkcs7,
                          const char* clientocspresponse,
                          const char* serverocspresponse );

/** bid_createdynamicsdo creates an unsealed SDO. The SDO will consist of all signatures applied in the function call.
    The data itself will not be embedded in the SDO. The merchant must call BID_SEIDSDOAddData to embed the data in the SDO.
    The SDO will not be sealed. The merchant must call BID_SealSDO to seal all signatures.
    @param sessioncontext This is the context handle for the current merchant
    @param sdo This a reference to a pointer to a BID_SEIDSDO structure where BankID C Server will allocate memory and fill in the SDO.
    @param data This is the data (the contract) itself.
    @param datatype The datatype of the signed data This must be either SDODATA_TEXT or SDODATA_PDF.
    @param datadescription The datadescription of the signed data.
    @param signatures The list of all signatures that will be embedded in the SDO.
    @return Returns zero if BankID C Server was able to create the Signed Data Object.
            If an error occurs during the contract building operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_createdynamicsdo( const int sessioncontext,
                                 BID_SEIDSDO** sdo,
                                 const char* data,
                                 const char* datatype,
                                 const char* datadescription,
                                 const BID_Signature* signatures);

/** bid_sdotoxmlex will take an SDO structure and create an XML string of it. The XML string can
    afterwards be saved as a regular textfile. The output is encoded as UTF-8.
    @param sessioncontext This is the context handle for the current merchant
    @param sdo This is the constructed SDO structure.
    @param sdoxml This is a reference to a pointer where BankID C Server will allocate memory and fill in the SDO XML.

    @return Returns zero if BankID C Server was able to construct the XML.
            If an error occurs during the construction operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_sdotoxmlex( const int sessioncontext,
                           const BID_SEIDSDO* sdo,
                           char** sdoxml);

/** bid_dynamicsdovalidate will validate a dynamic SDO.
    @param sessioncontext This is the context handle for the current merchant
    @param sdo This is the constructed SDO structure.
    @param data This is the data (the contract) itself.
    @param numsignatures The expected number of signatures in the SDO.
    @param sealcheck 0: Must be present and verified 1: Must not be present, 2: Can be present, verify if present.

    @return Returns zero if BankID C Server was able to verify the SDO.
            If an error occurs during the verification operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_dynamicsdovalidate( const int sessioncontext,
                                   const BID_SEIDSDO* sdo,
                                   const char* data,
                                   const int numsignatures,
                                   const int sealcheck );


/** bid_sdovalidate will validate an SDO. If the data itself should be verified the verifydata flag must be set.
    @param sessioncontext This is the context handle for the current merchant
    @param sdo This is the constructed SDO structure.
    @param verifydata If ==0, the data in the SDO will not be verified.
                      If <>0, the data (BID_SDOAddData() must previously have been called) will be verified.

    @return Returns zero if BankID C Server was able to verify the SDO.
            If an error occurs during the verification operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_sdovalidate(const int sessioncontext,
                           const BID_SEIDSDO* sdo,
                           const int verifydata);

/** bid_sdoxmlvalidate will validate an SDO. If the data itself should be verified the verifydata flag must be set.
    @param sessioncontext This is the context handle for the current merchant
    @param sdo This is the constructed SDO structure.
    @param verifydata If ==0, the data in the SDO will not be verified.
                      If <>0, the data (BID_SDOAddData() must previously have been called) will be verified.

    @return Returns zero if BankID C Server was able to verify the SDO.
            If an error occurs during the verification operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_sdoxmlvalidate(const int sessioncontext,
                              const char* sdo,
                              const int verifydata);


/** bid_sdofree frees all memory absorbed by the SEIDSDO structure.
    @param sessioncontext This is the context handle for the current merchant
    @param sdo This is the constructed SEIDSDO structure.

    @return Returns zero if BankID C Server was able to release the memory absorbed by the SDO
            If an error occurs during the free operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int  bid_sdofree(const int sessioncontext,
                         BID_SEIDSDO* sdo);

/** bid_signaturesfree frees all memory allocated by a BID_Signature structure.
 * @param sessioncontext This is the context handle to be used.
 * @param signature This is the BID_Signature structure to free
 *
 * @return Returns zero on success
 *         If an error occurs during the free operation an errorcode is returned.
 *         The errorcode is documented in the BankID C Server Interface Specification.
 */
extern int bid_signaturesfree(const int sessioncontext, BID_Signature *signature);

/** bid_sdoadddata adds a contract to the SDO.
    @param sessioncontext This is the context handle for the current merchant.
    @param sdo This is the SDO that should contain the contract.
    @param data The contract to add base64 encoded.

    @return Returns zero if BankID C Server was able to add the contract.
            If an error occurs during the sdoadddata operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_sdoadddata(const int sessioncontext,
                          BID_SEIDSDO* sdo,
                          const char* data);


/** BID_SDOSeal seals an SDO.
    @param sessioncontext This is the context handle for the current merchant.
    @param sdo  This is the SDO to be sealed
    @param ocsp A newly created OCSP response for the merchant itself
    @param data The contract that has been signed in the SDO

    @return Returns zero if BankID C Server was able to add the contract.
            If an error occurs during the sdoadddata operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_sdoseal(const int sessioncontext,
                       BID_SEIDSDO* sdo,
                       const char* ocsp,
                       const char* data);


/** bid_sdoaddsignature adds signatures to an existing SDO
    @param sessioncontext This is the context handle for the current merchant.
    @param sdo This is the SDO that should contain the contract.
    @param signatures All signatures that sould be put into the SDO.
    @param data The contract that is signed.

    @return Returns zero if BankID C Server was able to add the signature(s).
            If an error occurs during the sdoaddsignature operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_sdoaddsignature(const int sessioncontext,
                               BID_SEIDSDO* sdo,
                               const BID_Signature* signatures,
                               const char* data);


/** bid_free frees memory allocated by BankID C Server.
    @param allocated This is a pointer to data previosly allocated by BankID C Sercer.

    @return Returns zero if BankID C Server was able to release the memory absorbed.
            If an error occurs during the free operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_free(void* allocated);


/** bid_getrecenterror gives the application a string describing the last error that occured in BankID Server.
    @param sessioncontext This is the context handle for the current merchant
    @param error This is a reference to a char pointer describing the very last error that occured in BankID Server.
*/
extern int bid_getrecenterror( const int sessioncontext,
                               char** error);


/** bid_xmltosdo takes an SDO XML previosly created by bid_sdotoxmlex() and parses it into an SDO structure.
    The sdoxml must be valid UTF-8 text.
    @param sessioncontext This is the context handle for the current merchant
    @param sdoxml This is the xml representation of an SDO.
    @param sdo This is the reference to the pointer that BankID C Server will fill with an SDO struct.
*/
extern int bid_xmltosdoex(const int sessioncontext,
                          const char* sdoxml,
                          BID_SEIDSDO** sdo);

/** bid_createsecurechannel is used by the merchant to create a message directly to the COI.
    The channel is at present used to inform the COI about the demands currently set on the client.
    If the merchant demands that the client enteres a new OTP for the current transaction this function,
    The messages (key,value) are set by calling BID_SetInfoItem prior to calling this method.
    @param sessioncontext This is the context handle for the current merchant
    @param pkcs7 The merchants own pkcs7 for the current transaction.
    @param channelpkcs7 This is a reference to a pointer where BankID C Server will allocate memory and fill in the signature over the channel.
    @param channeldate This is a reference to a pointer where BankID C Server will allocate memory and fill in the channel data.

    @return Returns zero if BankID C Server was able to release the memory absorbed.
            If an error occurs during the free operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_createsecurechannel( const int sessioncontext,
                                    const char* pkcs7,
                                    char** channelpkcs7,
                                    char** channeldata);


/** bid_getcertstatuswithnonce accepts a PKCS#7 structure, retrieves the signing certificate and requests a certificate status 
    towards the Validation Authority. If a pcks#7 is not present in the call, a VA lookup on the merchant itself is done.
    @param sessioncontext This is the context handle for the current merchant
    @param pkcs7 This is the BASE64 encoded signature in which the certificate in request resides.
    @param ocspresponse This is a reference to a pointer where BankID C Server will allocate storage and fill in the OCSP response from the VA.
    @param nonce This is a reference to a pointer where BankID C Server will allocate storage and fill in the OCSP request nonce sent to the VA.
                        If the parameter is NULL, the OCSP response will not be supplied.

    @return Returns zero if BankID C Server successfully called getcertstatus.
            If an error occurs during the getcertstatus operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_getcertstatuswithnonce( const int sessioncontext,
                                       const char* pkcs7,
                                       char** ocspresponse,
                                       char** nonce);

/** bid_assymetricdecrypt decrypts PKCS#8 encoded data using the passed private key.
    @param sessioncontext This is the context handle for the current merchant
    @param keylocation This is the location of the PKCS#8 private key.
    @param password The password for the PKCS#8 private key.
    @param BASE64 encoded encrypted data
    @param BASE64 encoded decrypted data
*/
extern int bid_assymetricdecrypt(const int sessioncontext,
                                 const char* keylocation,
                                 const char* password,
                                 const char* encrypted,
                                 char** decrypted);

/** bid_getsessionstoreinfo extracts key information about all registered sessions for the BankID Server runtime.
    @param sessioncontext This is the context handle for the current merchant.
    @param sessioninfo this is a linked list of all registered sessions with session information.

    @return Returns zero if BankID C Server was able to add the signature(s).
            If an error occurs during the sdoaddsignature operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_getsessionstoreinfo( const int sessioncontext,
                                    BID_SessionInfo** sessioninfo);

/** bid_freesessionstoreinfo freees the sessionstoreinfo structure list.
    @param sessioncontext This is the context handle for the current merchant.
    @param sessioninfo The linked list to free.

    @return Returns zero if BankID C Server was able to free the data.
            If an error occurs during the freesessionstore operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_freesessionstoreinfo(const int sessioncontext,
                                    BID_SessionInfo* sessioninfo);

/** bid_requestmobileaction is used by the merchant to perform a BankID operation using the mobile phone of the user.
    The parameteres must be set in the merchants session prior to calling this method by using bid_setinfoitem().
    These parameters include the action, the URL for the communication, the phone number and other parameters.

    @param sessioncontext This is the context handle for the current merchant
    @param transactionreference This is a transaction reference for the performed operation. This can later be used
                                for sending the user an SMS using bid_pushsms
                     
    @return Returns zero if mobile user successfully performed the action
            If an error occurs during the mobile operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_requestmobileaction( const int sessioncontext,
                                    char** transactionreference);


/** bid_pushsms can be used to send the mobile user an SMS upon receiving a transaction reference from bid_requestmobileaction
    
    @param sessioncontext This is the context handle for the current merchant
    @param transactionreference The reference returned after a successfull mobile action
    @param text The text to send to the mobile user
                     
    @return Returns zero if the SMS request was delivered to the MGMFE succesfully
            If an error occurs during the communication an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_pushsms( const int sessioncontext,
                         const char* transactionreference,
                        const char* text );


/** bid_getsigners retrieves the signers from the linked list of signatures. The output string is semicolon separated 
    and used by the netcentric client during the multisigning process.

    @param sessioncontext This is the context handle for the current merchant.
    @param signatures The linked list of signatures to be parsed
    @param signers The string containing information about the signers

    @return Returns zero if BankID C Server was able to retrieve the signer informations
            If an error occurs during the bid_getsigners operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/ 
extern int bid_getsigners( const int sessioncontext, 
                           BID_Signature* signatures, 
                           char** signers );


/** bid_sdogetsigners retrieves the signers of the SEIDSDO SDO structure. The output string is semicolon separated 
    and used by the netcentric client during the multisigning process.

    @param sessioncontext This is the context handle for the current merchant.
    @param sdo This is the SDO that should contain the singatures
    @param signers The string containing information about the signers

    @return Returns zero if BankID C Server was able to retrieve the signer informations
            If an error occurs during the bid_sdogetsigners operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/ 
extern int bid_dynamicsdogetsigners( const int sessioncontext,
                                     const BID_SEIDSDO* sdo,
                                     char** signers );


/** bid_sdoxmlgetsigners retrieves the signers of the SEIDSDO SDO XML. The output string is semicolon separated 
    and used by the netcentric client during the multisigning process.

    @param sessioncontext This is the context handle for the current merchant.
    @param sdo This is the SDO XML that should contain the singatures
    @param signers The string containing information about the signers

    @return Returns zero if BankID C Server was able to retrieve the signer informations
            If an error occurs during the bid_sdoxmlgetsigners operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/ 
extern int bid_dynamicsdoxmlgetsigners( const int sessioncontext,
                                        const char* sdo,
                                        char** signers);

/** bid_simpledynamicsdoxmlvalidate will do a simple validation of a dynamic SDO. It only checks that the dynamic
    SDO is sealed and contains at least n signatures.
    @param sessioncontext This is the context handle for the current merchant
    @param sdo This is the SDO XML that should contain the singatures.
    @param minnumsignatures The minimum number of signatures expected in the SDO.

    @return Returns zero if BankID C Server was able to verify the SDO.
            If an error occurs during the verification operation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_simpledynamicsdoxmlvalidate(const int sessioncontext,
                                           const char* sdo,
                                           const int minnumsignatures );

/** bid_requestmobilestatus retrieves the status from an earlier initialized 
    bid_requestmobileaction.
    @param sessioncontext This is the context handle for the current merchant
    @param transactionreference The reference returned after a successfull mobile action
    @param transactionstatus The transaction status

      @return Returns zero if the status was retrieved and is OK
            If an error occurs during the communication, or the status is not OK
            an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_requestmobilestatus( const int sessioncontext,
                                    const char* transactionreference,
                                    char** transactionstatus);

/** bid_generatemerchantreference generates the reference to show to the
    user during mobile authentication.

    @param sessioncontext This is the context handle for the current merchant
    @param locale The locale to use. Supported locales: no_NO, en_GB
    @param merchantreference This is the text to be displayed to the user.

    @return Return zero if everything is supagreen.
            If an error occurs during the reference generation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_generatemerchantreference( int sessioncontext,
                                          const char *locale,
                                          char **merchantreference);

/** bid_decrypttransactiondata can be used by the merchant to decrypt the transaction data used during
    BankID communication, both the request from the client and the response genererated by
    BankID Server.
    
    @param sessioncontext This is the context handle for the current merchant
    @param enckey The encryption key
    @param encdata The encrypted data
    @param operation The operation
    @param sid The sid
    @param data The decrypted data
    @param key The encryption key
                     
    @return Returns zero if the decryption was successfull
            If an error occurs during the dedcryption an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification.
*/
extern int bid_decrypttransactiondata( const int sessioncontext,
                                       const char* enckey,
                                       const char* encdata,
                                       const char* encauth,
                                       const char* operation,
                                       const char* sid,
                                       char** data,
                                       char** key );


/** bid_inittransaction is used by the merchant to create the encrypted response to the initAuth/Sign operation 
    from the encrypted request of the BankID client. In addition to the ecnrypted response, a number of infoitems
    are available using bid_getinfoitem(). See the the BankID C Server Interface Specification for the details.

    @param sessioncontext This is the context handle for the current merchant
    @param enckey The encryption key
    @param encdata The encrypted request
    @param enchauth The MAC over the encrypted data
    @param operation The operation parameter sent from the BankID Client
    @param sid The sid parameter sent from the BankID Client.
    @param response The encrypted response
                     
    @return Returns zero if the encrypted response was created succsessfully.
            If an error occurs during the response creation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification
*/
extern int bid_inittransaction( const int sessioncontext,
                                const char* enckey,
                                const char* encdata,
                                const char* encauth,
                                const char* operation,
                                const char* sid,
                                char** response);


/** bid_verifytransactionrequest is used by the merchant to handle the verifyAuth/verifySign/handleError operation
    from the encrypted request of the BankID client. In addition to the encrypted request, some infoitems
    must be set prior to calling this method using bid_setinfoitem(). Some infoitems will also be available using
    bid_getinfoitem() after calling this method. See the the BankID C Server Interface Specification for the details.
    
    @param sessioncontext This is the context handle for the current merchant
    @param enckey The encryption key
    @param encdata The encrypted request
    @param enchauth The MAC over the encrypted data
    @param operation The operation parameter sent from the BankID Client
    @param sid The sid parameter sent from the BankID Client.                     
    @return Returns zero if the verification of the request was successfull.
            If an error occurs during the dedcryption an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification
*/
extern int bid_verifytransactionrequest( const int sessioncontext,
                                         const char* enckey,
                                         const char* encdata,
                                         const char* encauth,
                                         const char* operation,
                                         const char* sid);


/** bid_verifytransactionresponse is used by the merchant to create the encrypted response for the 
    verifyAuth/verifySign/handleError operation or sending an error message should an error occur.
    Some infoitems    must be set prior to calling this method using bid_setinfoitem(). See the the 
    BankID C Server Interface Specification for the details.
    
    @param sessioncontext This is the context handle for the current merchant
    @param response The encrypted response
                     
    @return Returns zero if the encrypted response was created succsessfully.
            If an error occurs during the response creation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification
*/
extern int bid_verifytransactionresponse( const int sessioncontext,
                                          char** response );

/** bid_sdoxmlextractsignatures is used by the merchant to extract all relevant data for sdo xml
    structures created by earlier versions of BankID Server.
    
    @param sessioncontext This is the context handle for the current merchant
    @param sdoxml The SDO XML
    @param data    The data in the sdo will be put here
    @param datatype    The datatype in the sdo will be put here 
    @param datadescription    The datadescription in the sdo will be put here   
    @param signatures    The signatures in the sdo will be put here

    @return Returns zero if the signatures were created successfully.
            If an error occurs during the response creation an errorcode is returned.
            The errorcode is documented in the BankID C Server Interface Specification
*/
extern int bid_sdoxmlextractsignatures( const int sessioncontext,
                                        const char* sdoxml,
                                        char** data,
                                        char** datatype,
                                        char** datadescription,
                                        BID_Signature** signatures);

/** bid_gettransactioninfo is used to request additional information about a transaction.
 *
 * @param sessioncontext This is the context handle for the current merchant
 * @param key This is the transaction id to request information for
 * @param oids This is a comma separated list of oids that identify the information the merchant is allowed to request
 * @param timeout This is the maximum time to wait for data to become ready
 *
 * @return Returns zero if the request was processed OK and a response was generated.
 *         If an error occurs an errorcode is returned.
 *         The errorcode is documented in the BankID C Server Interface Specification.
 */
extern int bid_gettransactioninfo( const int sessioncontext, const char *key, const char *oids, const char *timeout);

/** bid_initsession is used to initialize a session between a merchant and a javascript client.
 *
 * The following infoitems must be set before this method is invoked:
 *  - useragent, url, timeout, sid, action
 *
 *  @param sessioncontext This is the context handle for the current merchant
 *  @param helperuri The uri to download the bankid helper from.
 *  @param traceid The traceid for this session.
 *  @param clientid The clientId to the generated BankID 2.0 client.
 *
 *  @return Returns zero on success, a errorcode if something went wrong.
 */
extern int bid_initsession(const int sessioncontext, char **helperuri, char **traceid, char **clientid);

/** bid_getversioninfo is used to get information about the BankID C Server library.
 *
 *  @param sessioncontext This is the context handle for the current merchant
 *  @param versioninfo The version information is returned in this char**
 *  @return Returns zero on success, a errorcode if something went wrong.
 */
extern int bid_getversioninfo(char **versioninfo);

/** bid_adddocumenttext is used to add a text document to be signed in BankID 2.1
 *
 * @param sessioncontext This is the context handle for the current merchant
 * @param data The bytes of the document to be signed
 * @param dataDescription The dataDescription for the document to be signed
 *
 * @return Returns zero on success, an errorcode if something went wrong.
 **/
extern int bid_adddocumenttext(const int sessioncontext, const char *data, const char *dataDescription);

/** bid_adddocumentxml is used to add a xml document to be signed in BankID 2.1
 *
 * @param sessioncontext This is the context handle for the current merchant
 * @param xml The xml bytes to be signed
 * @param xsl The xsl transformation to be applied to the xml (for viewing)
 * @param dataDescription The description of the document
 *
 * @return Returns zero on success, an errorcode if something went wrong.
 */
extern int bid_adddocumentxml(const int sessioncontext, const char *xml, const char *xsl, const char *dataDescription);

/** bid_adddocumentpdf is used to add a pdf document to be signed in BankID 2.1
 *
 * @param sessioncontext This is the context handle for the current merchant
 * @param data The bytes of the pdf to be signed.
 * @param dataDescription The description of the document
 *
 * @return Returns zero on success, an errorcode if something went wrong.
 */
extern int bid_adddocumentpdf(const int sessioncontext, const char *data, const char *dataDescription);

/** bid_getsigneddata is used to get the signed data and the merchant signature for a signed document in BankID 2.1
 *
 * @param sessioncontext This is the context handle for the current merchant
 * @param docnum The number of the document to fetch signed data for. 0 is the first document.
 * @param signed_data The data was signed. 
 * @param signature The merchant pkcs7 for this data.
 *
 * @return Returns zero on success, an errorcode if something went wrong.
 */
extern int bid_getsigneddata(const int sessioncontext, const int docnum, char **signed_data, char **signature);

/** bid_setsigneddata is used to set the data that was signed before a call to bid_verifytransactionrequest.
 *
 * @param sessioncontext This is the context handle for the current merchant
 * @param data The signed data (obtained with bid_getsigneddata after a successful call to bid_inittransaction)
 *
 * @return Returns zero on success, an errorcode if something went wrong.
 */
extern int bid_setsigneddata(const int sessioncontext, const char *data);

/** bid_getsignatureandocsp is used to get client signature and ocsp after a call to bid_verifytransactionrequest.
 *
 * @param sessioncontext This is the context handle for the current merchant
 * @param docnum The document number to fetch signature and ocsp for. 0 is the first document.
 * @param signature The client signature is returned here
 * @param ocsp The client ocsp is returned here
 *
 * @return Returns zero on success, an errorcode if something went wrong.
 */
extern int bid_getsignatureandocsp(const int sessioncontext, const int docnum, char **signature, char **ocsp);
/** bid_getsignatureandocsp is used to get client signature and full ocsp after a call to bid_verifytransactionrequest.
*
* @param sessioncontext This is the context handle for the current merchant
* @param docnum The document number to fetch signature and ocsp for. 0 is the first document.
* @param signature The client signature is returned here
* @param ocsp The client ocsp is returned here
*
* @return Returns zero on success, an errorcode if something went wrong.
*/
extern int bid_getsignatureandfullocsp(const int sessioncontext, const int docnum, char **signature, char **fullocsp);

/** bid_getreportdata is used to fetch the new report elements from bankid server.
 *
 * @param sessioncontext This is the context handle for the current merchant
 * @param key The key to request data for.
 * @param out The data is returned here.
 *
 * @return Returns zero on success, an errorcode if something went wrong.
 */
extern int bid_getreportdata(const int sessioncontext, const char *key, char **out);

#ifdef __cplusplus
}
#endif


#endif

