package no.bbs.bankid.bidcserver.loaders;

import java.io.File;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Collections;
import java.util.HashSet;
import java.util.Set;

/**
 * Class which resolved names and directories inside package jar.
 *
 * Created by lkale on 27.10.2015.
 */
public class PackageClibResolver {

    public String findInJarDirectory() {
        Os os = Os.findCurrent();
        Arch arch = Arch.findCurrent();
        return "clib/" + os.name() + arch.getName();
    }

    public Set<String> listLibs() {
        Set<String> files = new HashSet<String>();
        Os os = Os.findCurrent();
        Arch arch = Arch.findCurrent();
        String osArchDir = os.name() + arch.getName();
        String inJarDirectory = "clib/" + osArchDir + "/";
        switch (osArchDir) {
            case "windows":
            case "windows_x86-64":
                files.add(inJarDirectory + "bidcserver.dll");
                files.add(inJarDirectory + "bidcserver.lib");
                files.add(inJarDirectory + "capi.dll");
                break;
            case "linux":
            case "linux_x86-64":
                files.add(inJarDirectory + "libbidcserver.so");
                files.add(inJarDirectory + "libCAPI.so");
                break;
            default:
                throw new IllegalStateException("Unknown in JAR directory: " + osArchDir);
        }
        return files;
    }

    /**
     * Enum which contains all operating systems.
     *
     * Created by lkale on 27.10.2015.
     */
    public enum Os {

        linux("Linux"),
        windows("Win");

        private final Set<String> osNames = new HashSet<String>();

        /**
         * Create os enum.
         *
         * @param osNames are possible values received from system properties.
         */
        Os(String ... osNames) {
            Collections.addAll(this.osNames, osNames);
        }

        public boolean isCurrent() {
            return isCurrent(System.getProperty("os.name"));
        }

        private boolean isCurrent(String osName) {
            boolean found = osNames.contains(osName);
            if (!found) {
                String firstName = osNames.iterator().next();
                found = osName.startsWith(firstName);
            }
            return found;
        }

        public static Os findCurrent() {
            final String osName = System.getProperty("os.name");
            for(Os os : values()) {
                if (os.isCurrent(osName)) {
                    return os;
                }
            }
            throw new IllegalStateException("Unsupported OS: " + osName);
        }
    }

    /**
     * Enum which contains all architectures.
     */
    public enum Arch {

        x86_32("",             "i386", "i686"),
        x86_64("_x86-64",      "amd64", "x86");

        private final String name;

        private final Set<String> osArchs = new HashSet<String>();

        /**
         * Create arch enum.
         *
         * @param name is an part of directory where library is stored.
         * @param osArch is an possible value received from system properties.
         */
        Arch(String name, String ... osArch) {
            this.name = name;
            Collections.addAll(this.osArchs, osArch);
        }

        public String getName() {
            return name;
        }

        public boolean isCurrent() {
            return isCurrent(System.getProperty("os.arch"));
        }

        private boolean isCurrent(String osName) {
            return osArchs.contains(osName);
        }

        public static Arch findCurrent() {
            final String osArch = System.getProperty("os.arch");
            for(Arch arch : values()) {
                if (arch.isCurrent(osArch)) {
                    return arch;
                }
            }
            throw new IllegalStateException("Unsupported ARCH: " + osArch);
        }

    }
}
